from django.utils.datastructures import MultiValueDict


def jquery_to_dict(values):
    result = MultiValueDict()
    if not isinstance(values, (list, tuple, set)):
        return result
    for value in values:
        if not isinstance(value, dict):
            continue
        if 'name' in value and 'value' in value:
            result.appendlist(value['name'], value['value'])
    return result



from functools import wraps
from django.http import HttpResponse


def render_to_json(func):
    @wraps(func)
    def _render_to_json(request, *args, **kwargs):
        from django_ajax.views import render_to_json as render_to_json_view

        response = func(request, *args, **kwargs)
        if isinstance(response, HttpResponse):
            return response
        return render_to_json_view(request, response)

    return _render_to_json




from django import template

try:
    import json as _json
except ImportError:
    import django.utils.simplejson as _json

register = template.Library()


class JsonNode(template.Node):
    def __init__(self, variable, or_null):
        self.variable = variable
        self.or_null = or_null

    def render(self, context):
        try:
            obj = self.variable.resolve(context)
        except template.VariableDoesNotExist:
            if self.or_null:
                return 'null'
            else:
                return ''
        if hasattr(obj, 'ajax_data'):
            obj = obj.ajax_data()
        return _json.dumps(obj)


@register.tag
def json(parser, token):
    '''
    {% json object %}
    {% json object or null %}
    '''
    tokens = token.split_contents()
    tag_name = tokens[0]
    values = tokens[1:]
    if len(values) not in (1, 3):
        raise template.TemplateSyntaxError(u'%r requires one or three argument.' % tag_name)
    # no compile_filter, as this does not throw VariableDoesNotExist
    variable = template.Variable(values[0])
    or_null = False
    if len(values) > 1:
        if values[1] != 'or':
            raise template.TemplateSyntaxError(u'%r second argument must be "or".' % tag_name)
        if values[2] != 'null':
            raise template.TemplateSyntaxError(u'%r third argument must be "null".' % tag_name)
        or_null = True
    return JsonNode(variable, or_null)


class AjaxCacheNode(template.Node):
    def __init__(self, variable, cache_func):
        self.variable = variable
        self.cache_func = cache_func

    def render(self, context):
        try:
            obj = self.variable.resolve(context)
            cache_func = self.cache_func.resolve(context)
        except template.VariableDoesNotExist:
            return ''
        if not hasattr(context, '_ajax_cached'):
            context._ajax_cached = {}
        if not cache_func in context._ajax_cached:
            context._ajax_cached[cache_func] = []
        if obj.pk in context._ajax_cached[cache_func]:
            return ''
        context._ajax_cached[cache_func].append(obj.pk)
        if hasattr(obj, 'ajax_data'):
            obj = obj.ajax_data()
        return u'%(cache_func)s(%(dump)s);' % {
            'cache_func': cache_func,
            'dump': _json.dumps(obj),
        }


@register.tag
def ajax_cache(parser, token):
    '''
    {% ajax_cache obj using "some_func.to.call" %}
    '''
    tokens = token.split_contents()
    tag_name = tokens[0]
    values = tokens[1:]
    if len(values) < 3:
        template.TemplateSyntaxError(u'%r requires at least one parameter.' % tag_name)
    variable = parser.compile_filter(values[0])
    if values[1] != 'using':
        template.TemplateSyntaxError(u"%r's second parameter must be 'using'." % tag_name)
    cache_func = template.Variable(values[2])
    return AjaxCacheNode(variable, cache_func)




from django.http import HttpResponse
from django.contrib.auth.forms import AuthenticationForm
from django.contrib.auth import login as auth_login
from django_ajax.decorators import render_to_json as render_to_json_decorator


def default_getstate(obj):
    try:
        return obj.__getstate__()
    except AttributeError:
        # TODO: Fix this, must raise exception somehow
        from django.utils.simplejson import JSONEncoder

        return JSONEncoder.default(None, obj)


def render_to_json(request, json_obj):
    try:
        import json
    except ImportError:
        import django.utils.simplejson as json

    json_string = json.dumps(json_obj, default=default_getstate)
    response = HttpResponse(json_string)
    # We support <iframe>-responses by allowing passing the iframe-param here.
    # <iframes> will not work in IE using the official mime type as the browser
    # will schow a "save file as" dialog. So <iframe>'s will be sent as
    # text/plain, which should work for other browsers, too.
    if request.REQUEST.get('iframe'):
        response['Content-Type'] = "text/plain; charset=utf-8"
    else:
        response['Content-Type'] = "application/json; charset=utf-8"
    response['Pragma'] = "no cache"
    response['Cache-Control'] = "no-cache, must-revalidate"
    return response


@render_to_json_decorator
def login(request, authentication_form=AuthenticationForm):
    status = 'ok'
    if request.method == "POST":
        form = authentication_form(data=request.POST)
        if form.is_valid():
            # Okay, security checks complete. Log the user in.
            auth_login(request, form.get_user())
            if request.session.test_cookie_worked():
                request.session.delete_test_cookie()
            return {
                'status': status,
                'authenticated': True,
                'session_key': request.session._session_key,
            }
        else:
            status = 'failed'
    else:
        form = authentication_form(request)
    request.session.set_test_cookie()
    return {
        'status': status,
        'authenticated': False,
    }


@render_to_json_decorator
def logout(request):
    from django.contrib.auth import logout

    logout(request)
    return {
        'status': 'ok',
        'authenticated': False,
    }




def media(request):
    from django.conf import settings

    return {
        'MEDIA_URL': settings.MEDIA_URL
    }


def static(request):
    from django.conf import settings

    return {
        'STATIC_URL': settings.STATIC_URL
    }


def session(request):
    from django.core.urlresolvers import reverse

    return {
        'session': {
            'authenticated': request.user.is_authenticated(),
            'urls': {
                'login': reverse('ajax_login'),
                'logout': reverse('ajax_logout'),
            }
        }
    }




# -*- coding: utf-8 -*-
from django.conf.urls import *

urlpatterns = patterns('django_ajax.views',
    url(r'^login/$', 'login', name='ajax_login'),
    url(r'^logout/$', 'logout', name='ajax_logout'),
)



from setuptools import setup, find_packages

setup(
    name = "django_ajax",
    version = "0.2.0",
    description = 'Utility app to support AJAX development in Django',
    author = 'David Danier',
    author_email = 'david.danier@team23.de',
    url = 'https://github.com/ddanier/django_ajax',
    long_description=open('README.rst', 'r').read(),
    packages = [
        'django_ajax',
        'django_ajax.templatetags',
    ],
    install_requires = [
        'Django >=1.4',
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Utilities'
    ],
)




from django.core.exceptions import ImproperlyConfigured
from django.utils.importlib import import_module
from django.conf import settings

AJAX_CONFIG_PROCESSORS = getattr(settings, 'AJAX_CONFIG_PROCESSORS', ())


# TODO: Cache loaded processors
def get_config_processors(collect=AJAX_CONFIG_PROCESSORS):
    processors = []
    for path in collect:
        i = path.rfind('.')
        module, attr = path[:i], path[i + 1:]
        try:
            mod = import_module(module)
        except ImportError, e:
            raise ImproperlyConfigured('Error importing ajax config processor module %s: "%s"' % (module, e))
        try:
            func = getattr(mod, attr)
        except AttributeError:
            raise ImproperlyConfigured(
                'Module "%s" does not define a "%s" callable ajax config processor' % (module, attr))
        processors.append(func)
    return processors


def js_context(request):
    processors = get_config_processors()
    context = {}
    for processor in processors:
        context.update(processor(request))
    return context

